# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log

from almanach.core import exception
from almanach.storage.drivers import mongodb_driver

LOG = log.getLogger(__name__)


class StorageDriver(object):
    def __init__(self, config):
        self.config = config

    def get_database_driver(self):
        if self.config.database.driver == 'mongodb':
            LOG.info('Loading MongoDb storage driver')
            return mongodb_driver.MongoDbDriver(self.config)
        raise exception.DatabaseDriverNotSupportedException(
                'Unknown database driver {}'.format(self.config.database.driver))
